<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;
use Util\NovoCodigo;
use Ems\Util\Utilities;

class ParceiroNegocioRepository {
    /** @var Connection */
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection           = $entityManager->getConnection();
        $this->entityManager        = $entityManager;
    }

    public function getParceiroNegocioByPerfilAcesso(int $nrorg, int $nrperfilacesso): Array {
        $sql = <<<SQL
            SELECT *
              FROM PERFACESSPARC PAP
              JOIN PARCNEGOCIO   PN  ON PN.NRORG = PAP.NRORG AND PN.NRPARCNEGOCIO = PAP.NRPARCNEGOCIO
             WHERE PAP.NRORG = :P_NRORG
                   AND PAP.NRPERFILACESSO = :P_NRPERFILACESSO
SQL;

        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_NRPERFILACESSO"  => $nrperfilacesso
            )
        );

        $result = $query->fetchAll();

        return $result;
    }

    public function getComunicacaoParc($nrorg, $nrvinculom) {
        $sql = <<<SQL
            SELECT
                TELEFONE_PRINCIPAL.CDCOMUNICAPARC   TELPRINCIPAL,
                TELEFONE_PRINCIPAL.CDPREFIXCOMUPARC PREFTELPRINCIPAL,
                TELEFONE_RESIDENCIAL.CDCOMUNICAPARC TELRESIDEN,
                TELEFONE_RESIDENCIAL.CDPREFIXCOMUPARC PREFTELRESIDEN,
                TELEFONE_COMERCIAL.CDCOMUNICAPARC TELCOMERC,
                TELEFONE_COMERCIAL.CDPREFIXCOMUPARC PREFTELCOMERC,
                CELULAR.CDCOMUNICAPARC CELPARTIC,
                CELULAR.CDPREFIXCOMUPARC PREFCELPARTIC,
                CELULAR_COMERCIAL.CDCOMUNICAPARC CELCOMERC,
                CELULAR_COMERCIAL.CDPREFIXCOMUPARC PREFCELCOMERC,
                FAX.CDCOMUNICAPARC PREFFAX,
                FAX.CDPREFIXCOMUPARC FAX,
                EMAIL.CDCOMUNICAPARC EMAIL
            FROM GPE_VINCULOM VM
                JOIN GPE_PESSOA PE
                  ON VM.NRPESSOA = PE.NRPESSOA
                  AND VM.NRORG = PE.NRORG
                /* TELEFONE PRINCIPAL*/
                LEFT JOIN COMUNICAPARC TELEFONE_PRINCIPAL
                      ON TELEFONE_PRINCIPAL.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                      AND TELEFONE_PRINCIPAL.NRORG = VM.NRORG
                      AND TELEFONE_PRINCIPAL.CDFORMACOMU = '01'
                      AND TELEFONE_PRINCIPAL.NRCOMUNICAPARC = (
                           SELECT MAX(NRCOMUNICAPARC) FROM COMUNICAPARC MAX_TP
                           WHERE MAX_TP.NRPARCNEGOCIO = TELEFONE_PRINCIPAL.NRPARCNEGOCIO
                           AND MAX_TP.NRORG = TELEFONE_PRINCIPAL.NRORG
                           AND MAX_TP.CDFORMACOMU = TELEFONE_PRINCIPAL.CDFORMACOMU
                    )

                /* CELULAR*/
                LEFT JOIN COMUNICAPARC CELULAR
                    ON CELULAR.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                    AND CELULAR.NRORG = VM.NRORG
                    AND CELULAR.CDFORMACOMU = '02'
                    AND CELULAR.NRCOMUNICAPARC = (
                            SELECT MAX(NRCOMUNICAPARC) FROM COMUNICAPARC MAX_CEL
                            WHERE MAX_CEL.NRPARCNEGOCIO = CELULAR.NRPARCNEGOCIO
                            AND MAX_CEL.NRORG = CELULAR.NRORG
                            AND MAX_CEL.CDFORMACOMU = CELULAR.CDFORMACOMU
                          )
                /* TELEFONE RESIDENCIAL*/
                LEFT JOIN COMUNICAPARC TELEFONE_RESIDENCIAL
                    ON TELEFONE_RESIDENCIAL.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                    AND TELEFONE_RESIDENCIAL.NRORG = VM.NRORG
                    AND TELEFONE_RESIDENCIAL.CDFORMACOMU = '03'
                    AND TELEFONE_RESIDENCIAL.NRCOMUNICAPARC = (
                            SELECT MAX(NRCOMUNICAPARC) FROM COMUNICAPARC CP
                            WHERE CP.NRPARCNEGOCIO = TELEFONE_RESIDENCIAL.NRPARCNEGOCIO
                            AND CP.NRORG = TELEFONE_RESIDENCIAL.NRORG
                            AND CP.CDFORMACOMU = TELEFONE_RESIDENCIAL.CDFORMACOMU
                          )
                /* TELEFONE COMERCIAL*/
                LEFT JOIN COMUNICAPARC TELEFONE_COMERCIAL
                    ON TELEFONE_COMERCIAL.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                    AND TELEFONE_COMERCIAL.NRORG = VM.NRORG
                    AND TELEFONE_COMERCIAL.CDFORMACOMU = '04'
                    AND TELEFONE_COMERCIAL.NRCOMUNICAPARC = (
                            SELECT MAX(NRCOMUNICAPARC) FROM COMUNICAPARC CP
                            WHERE CP.NRPARCNEGOCIO = TELEFONE_COMERCIAL.NRPARCNEGOCIO
                            AND CP.NRORG = TELEFONE_COMERCIAL.NRORG
                            AND CP.CDFORMACOMU = TELEFONE_COMERCIAL.CDFORMACOMU
                          )
                /* CELULAR COMERCIAL*/
                LEFT JOIN COMUNICAPARC CELULAR_COMERCIAL
                    ON CELULAR_COMERCIAL.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                    AND CELULAR_COMERCIAL.NRORG = VM.NRORG
                    AND CELULAR_COMERCIAL.CDFORMACOMU = '06'
                    AND CELULAR_COMERCIAL.NRCOMUNICAPARC = (
                            SELECT MAX(NRCOMUNICAPARC) FROM COMUNICAPARC CP
                            WHERE CP.NRPARCNEGOCIO = CELULAR_COMERCIAL.NRPARCNEGOCIO
                            AND CP.NRORG = CELULAR_COMERCIAL.NRORG
                            AND CP.CDFORMACOMU = CELULAR_COMERCIAL.CDFORMACOMU
                          )
                /* FAX*/
                LEFT JOIN COMUNICAPARC FAX
                    ON FAX.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                    AND FAX.NRORG = VM.NRORG
                    AND FAX.CDFORMACOMU = '10'
                    AND FAX.NRCOMUNICAPARC = (
                            SELECT MAX(NRCOMUNICAPARC) FROM COMUNICAPARC CP
                            WHERE CP.NRPARCNEGOCIO = FAX.NRPARCNEGOCIO
                            AND CP.NRORG = FAX.NRORG
                            AND CP.CDFORMACOMU = FAX.CDFORMACOMU
                          )
                /* EMAIL */
                LEFT JOIN COMUNICAPARC EMAIL
                    ON EMAIL.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                    AND EMAIL.NRORG = VM.NRORG
                    AND EMAIL.CDFORMACOMU = '05'
                    AND EMAIL.NRCOMUNICAPARC = (
                            SELECT MAX(NRCOMUNICAPARC) FROM COMUNICAPARC MAX_EMAIL
                            WHERE MAX_EMAIL.NRPARCNEGOCIO = EMAIL.NRPARCNEGOCIO
                            AND MAX_EMAIL.NRORG = EMAIL.NRORG
                            AND MAX_EMAIL.CDFORMACOMU = EMAIL.CDFORMACOMU
                       )
                WHERE VM.NRORG = :P_NRORG
                  AND VM.NRVINCULOM = :P_NRVINCULOM
SQL;

        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"       => $nrorg,
                "P_NRVINCULOM"  => $nrvinculom
            )
        );

        $result = $query->fetchAll();

        return $result[0];
    }

    public function getContaCorrenteParc($nrorg, $nrvinculom) {
        $sql = <<<SQL
            SELECT CC.CDBANCO,
                  BAN.NMBANCO,
                  CC.CDAGENCIA,
                  AG.NMAGENCIA,
                  CC.CDCONTCORR,
                  CC.NRTPCONTCORR,
                  TPCC.DSTPCONTCORR
            FROM GPE_VINCULOM VM
                JOIN GPE_PESSOA PE
                  ON VM.NRPESSOA = PE.NRPESSOA
                  AND VM.NRORG = PE.NRORG
                LEFT JOIN CONTCORRPARC CC ON CC.NRORG          = VM.NRORG
                                     AND CC.NRPARCNEGOCIO  = PE.NRPARCNEGOCIO /*parcnegocio da origem da cc*/
                                     AND CC.IDATIVO        = 'S'
            						             AND CC.NRCONTCORRPARC = ( SELECT MAX(NRCONTCORRPARC)
                                                               FROM CONTCORRPARC
                                                               WHERE NRORG         = CC.NRORG
                                                                 AND NRPARCNEGOCIO = CC.NRPARCNEGOCIO
                                                                 AND CC.IDATIVO        = 'S' )
            LEFT JOIN AGENCIA AG ON ( AG.NRORG = 0 OR AG.NRORG IS NULL OR AG.NRORG = VM.NRORG )
                                AND AG.CDBANCO   = CC.CDBANCO
                                AND AG.CDAGENCIA = CC.CDAGENCIA
            LEFT JOIN BANCO BAN ON BAN.CDBANCO = CC.CDBANCO
            LEFT JOIN TPCONTCORR TPCC ON TPCC.NRTPCONTCORR = CC.NRTPCONTCORR
                WHERE VM.NRORG = :P_NRORG
                  AND VM.NRVINCULOM = :P_NRVINCULOM
SQL;

        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"       => $nrorg,
                "P_NRVINCULOM"  => $nrvinculom
            )
        );

        $result = $query->fetchAll();

        return $result[0];
    }

    public function getEnderecoParc($nrorg, $nrvinculom) {
        $sql = <<<SQL
            SELECT EN.*,
                   PAIS.NMPAIS,
                   EST.NMESTADO,
                   MUN.NMMUNICIPIO,
                   LG.DSLOGRADOURO,
                   TE.NMTIPOENDERECO
            FROM GPE_VINCULOM VM
                JOIN GPE_PESSOA PE
                  ON VM.NRPESSOA = PE.NRPESSOA
                  AND VM.NRORG = PE.NRORG
                LEFT JOIN ENDERECOPARC EN ON EN.NRORG          = VM.NRORG
                                     AND EN.NRPARCNEGOCIO  = PE.NRPARCNEGOCIO /*parcnegocio da origem da cc*/
                                     AND EN.IDATIVO        = 'S'
                                     AND EN.CDTIPOENDERECO = 'PRINCIPAL'
                                     AND EN.NRENDERECOPARC = ( SELECT MAX(NRENDERECOPARC)
                                                               FROM ENDERECOPARC
                                                               WHERE NRORG         = EN.NRORG
                                                                 AND NRPARCNEGOCIO = EN.NRPARCNEGOCIO
                                                                 AND IDATIVO        = 'S'
                                                                 AND EN.CDTIPOENDERECO = 'PRINCIPAL' )
            LEFT JOIN
                TIPOENDERECO TE
                    ON TE.CDTIPOENDERECO = EN.CDTIPOENDERECO
                    AND(
                        EN.NRORG = TE.NRORG
                        OR TE.NRORG = 0
                    )
            LEFT JOIN
                LOGRADOURO LG
                    ON LG.CDLOGRADOURO = EN.CDLOGRADOURO
                    AND (
                        LG.NRORG = EN.NRORG
                        OR LG.NRORG = 0
                    )
            LEFT JOIN
                PAIS
                    ON PAIS.CDPAIS = EN.CDPAIS
            LEFT JOIN
                ESTADO EST
                    ON EST.SGESTADO = EN.SGESTADO
                    AND PAIS.CDPAIS = EST.CDPAIS
            LEFT JOIN
                MUNICIPIO MUN
                    ON MUN.CDMUNICIPIO = EN.CDMUNICIPIO
                    AND EST.SGESTADO = MUN.SGESTADO
                WHERE VM.NRORG = :P_NRORG
                  AND VM.NRVINCULOM = :P_NRVINCULOM
SQL;

        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"       => $nrorg,
                "P_NRVINCULOM"  => $nrvinculom
            )
        );

        $result = $query->fetchAll();

        return $result[0];
    }


    public function setParceiroNegocio(
        array $params
    ) {
        $sql = <<<SQL
            INSERT INTO PARCNEGOCIO (
                NRPARCNEGOCIO,      NRORG,              CDSEGMER,
                NMPRINCIPALPARC,    NMSECUNDARIPARC,    DTNASCIFUNDPARC,
                NRINSCRICAOPARC,    IDATIVO,            DTINCLUSAO,
                DTULTATU,           NRORGINCLUSAO,      CDOPERINCLUSAO,
                NRORGULTATU,        CDOPERULTATU,       CDTIPOPARCPRINCIPAL,
                CDTIPOINSCRICAO,    IDPESSOAFISICA,     IDINSTITUICAO,
                DSDEPARTAMENTO,     IDIDIOMA,           CDATIVCLI,
                NRSEQTPATIVCLI,     IDPARCFUNDIDO,      CDOPERRESPONSAVEL,
                NRTPCLASSIFICAPARC, DSDIRLOGOMARCA
            )
            VALUES (
                :NRPARCNEGOCIO,      :NRORG,              :CDSEGMER,
                :NMPRINCIPALPARC,    :NMSECUNDARIPARC,    :DTNASCIFUNDPARC,
                :NRINSCRICAOPARC,    :IDATIVO,            :DTINCLUSAO,
                :DTULTATU,           :NRORGINCLUSAO,      :CDOPERINCLUSAO,
                :NRORGULTATU,        :CDOPERULTATU,       :CDTIPOPARCPRINCIPAL,
                :CDTIPOINSCRICAO,    :IDPESSOAFISICA,     :IDINSTITUICAO,
                :DSDEPARTAMENTO,     :IDIDIOMA,           :CDATIVCLI,
                :NRSEQTPATIVCLI,     :IDPARCFUNDIDO,      :CDOPERRESPONSAVEL,
                :NRTPCLASSIFICAPARC, :DSDIRLOGOMARCA
            )
SQL;

            $std = $this->connection->prepare($sql);
            $std->execute($params);
    }

    public function setEnderecoParc(
        array $params
    ) {
        $sql = <<<SQL
            INSERT INTO ENDERECOPARC (
                NRENDERECOPARC,     NRPARCNEGOCIO,      CDTIPOENDERECO,
                NRORG,              IDATIVO,            DTINCLUSAO,
                DTULTATU,           NRORGINCLUSAO,      CDOPERINCLUSAO,
                NRORGULTATU,        CDOPERULTATU,       CDPAIS,
                SGESTADO,           CDMUNICIPIO,        CDBAIRRO,
                CDLOGRADOURO,       NMBAIRROENDERECO,   DSCOMPLEENDERECO,
                NRCEPENDERECO,      DSREFERENCIAENDE,   DSENDERECO,
                NRIMOVELENDERECO,   DSLATITUDE,         DSLONGITUDE
            )
            VALUES (
                :NRENDERECOPARC,    :NRPARCNEGOCIO,     :CDTIPOENDERECO,
                :NRORG,             :IDATIVO,           :DTINCLUSAO,
                :DTULTATU,          :NRORGINCLUSAO,     :CDOPERINCLUSAO,
                :NRORGULTATU,       :CDOPERULTATU,      :CDPAIS,
                :SGESTADO,          :CDMUNICIPIO,       :CDBAIRRO,
                :CDLOGRADOURO,      :NMBAIRROENDERECO,  :DSCOMPLEENDERECO,
                :NRCEPENDERECO,     :DSREFERENCIAENDE,  :DSENDERECO,
                :NRIMOVELENDERECO,  :DSLATITUDE,        :DSLONGITUDE
            )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function setContaCorrente(
        array $params
    ) {
        $sql = <<<SQL
            INSERT INTO CONTCORRPARC (
                NRCONTCORRPARC,     NRORG,          NRPARCNEGOCIO,
                CDREDUCONT,         CDBANCO,        CDAGENCIA,
                CDCONTCORR,         NRTPCONTCORR,   DTINCLUSAO,
                DTULTATU,           NRORGINCLUSAO,  CDOPERINCLUSAO,
                NRORGULTATU,        CDOPERULTATU,   IDATIVO
            )
            VALUES (
                :NRCONTCORRPARC,    :NRORG,         :NRPARCNEGOCIO,
                :CDREDUCONT,        :CDBANCO,       :CDAGENCIA,
                :CDCONTCORR,        :NRTPCONTCORR,  :DTINCLUSAO,
                :DTULTATU,          :NRORGINCLUSAO, :CDOPERINCLUSAO,
                :NRORGULTATU,       :CDOPERULTATU,  :IDATIVO
            )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    public function setComunicaParc(
        array $params
    ) {
        $sql = <<<SQL
            INSERT INTO COMUNICAPARC (
                NRCOMUNICAPARC,     NRPARCNEGOCIO,      CDFORMACOMU,
                NRENDERECOPARC,     CDPREFIXCOMUPARC,   CDCOMUNICAPARC,
                NRORG,              IDATIVO,            DTINCLUSAO,
                DTULTATU,           NRORGINCLUSAO,      CDOPERINCLUSAO,
                NRORGULTATU,        CDOPERULTATU,       CDCOMPLECOMUPARC,
                DSCONTATO
            )
            VALUES (
                :NRCOMUNICAPARC,    :NRPARCNEGOCIO,     :CDFORMACOMU,
                :NRENDERECOPARC,    :CDPREFIXCOMUPARC,  :CDCOMUNICAPARC,
                :NRORG,             :IDATIVO,           :DTINCLUSAO,
                :DTULTATU,          :NRORGINCLUSAO,     :CDOPERINCLUSAO,
                :NRORGULTATU,       :CDOPERULTATU,      :CDCOMPLECOMUPARC,
                :DSCONTATO
            )
SQL;

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     *  Busca por todos os parceiros de negócio disponíveis
     * 
     *  @param  string  $nrorg
     *  @param  string  $dtmescompetenc
     *  @param  string  $existeVinculos
     * 
     *  @return Array
     * 
     */
    public function getParcNegocioTodos(
        string $nrorg,
        string $dtmescompetenc,
        string $existeVinculos = null
    ) {
        $sql = <<<SQL
            SELECT DISTINCT
                PARC.*,
                PE.NRPESSOA
            FROM
                PARCNEGOCIO    PARC
                LEFT JOIN GPE_PESSOA     PE ON PARC.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                                          AND PARC.NRORG = PE.NRORG
                LEFT JOIN GPE_VINCULOM   VM ON PE.NRPESSOA = VM.NRPESSOA
                                             AND PE.NRORG = VM.NRORG
            WHERE
                PARC.NRORG = :P_NRORG
                AND ( ( :EXISTE_VINCULOS = 'S'
                        AND ( VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC
                              OR VM.DTRESCISAOVINC IS NULL ) ) OR ( :EXISTE_VINCULOS = 'N'
                          AND VM.DTRESCISAOVINC IS NOT NULL
                          AND VM.DTRESCISAOVINC < :P_DTMESCOMPETENC )
                      OR :EXISTE_VINCULOS IS NULL )
            ORDER BY PARC.NMPRINCIPALPARC
SQL;

        $query = $this->connection->prepare($sql);

        $query->execute(
            array(
                "P_NRORG"           => $nrorg,
                "P_DTMESCOMPETENC"  => $dtmescompetenc,
                "EXISTE_VINCULOS"   => $existeVinculos,
            )
        );

        $result = $query->fetchAll();

        return $result;
    }
}